<?php

namespace Drupal\field_context\Plugin\views\argument_default;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\node\NodeInterface;
use Drupal\views\Attribute\ViewsArgumentDefault;
use Drupal\views\Plugin\views\argument_default\ArgumentDefaultPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Default argument plugin to extract current page matching field.
 */
#[ViewsArgumentDefault(
  id: 'fcmatch',
  title: new TranslatableMarkup('Field from route context'),
)]
class FieldContext extends ArgumentDefaultPluginBase implements CacheableDependencyInterface {

  /**
   * Constructs a new instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Routing\RouteMatchInterface $routeMatch
   *   The route match.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity type manager.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected RouteMatchInterface $routeMatch,
    protected EntityFieldManagerInterface $entityFieldManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_route_match'),
      $container->get('entity_field.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    $options = parent::defineOptions();
    // Appears to just be defaults for the fields in buildOptionsForm.
    $options['fcftype'] = ['default' => ''];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    $field_map = $this->entityFieldManager->getFieldMap()['node'];
    // Stand the map on its head.
    $field_options = [];
    foreach ($field_map as $field_key => $data) {
      foreach ($data['bundles'] as $b) {
        $field_options[$b][$field_key] = $field_key;
      }
    }

    // Explicitly build [machine_name => machine_name] array.
    $options = array_combine(array_keys($field_options), array_keys($field_options));

    // Start with narrowing the list scope.
    $form['fcftype'] = [
      '#type' => 'select',
      '#title' => $this->t('Choose source content'),
      '#options' => array_merge(['' => '-Select-'], $options),
      '#default_value' => $this->options['fcftype'],
    ];

    // Ajax function inside this class is not callable, use #states to imitate.
    foreach ($field_options as $b => $f) {
      $form['fc' . $b] = [
        '#type' => 'select',
        '#title' => $this->t('Choose the field from @b', ['@b' => $b]),
        '#options' => $f,
        '#default_value' => $this->options['fc' . $b],
        '#states' => [
          // Show this field only if the Type is this bundle.
          'visible' => [
            // Use css selector.
            ':input[name="options[argument_default][fcmatch][fcftype]"]' => ['value' => $b],
          ],
        ],
      ];
    }

    // No return line in user or taxonomy files, which I am following.
  }

  /**
   * {@inheritdoc}
   */
  public function getArgument() {
    // Where the work happens.
    $key = $this->options['fcftype'];
    if (!empty($this->options['fc' . $key])) {
      // There is something to look for.
      // Just check, if a node could be detected.
      if (($node = $this->routeMatch->getParameter('node')) && $node instanceof NodeInterface) {
        // We're in, now look for chosen field.
        $source = $node->getFieldDefinitions();
        if (isset($source[$this->options['fc' . $key]])) {
          // This node has this field!
          return $node->get($this->options['fc' . $key])->getString();
        }
      }
    }
    // No return statement for catch-all fail case?
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge(): int {
    return Cache::PERMANENT;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts(): array {
    return ['url'];
  }

}
